from .stages import Stage


class URL:
    PREFIX = {
        Stage.PRODUCTION: "api",
        Stage.TEST: "stageapi",
    }
    BASE_FORMAT = "https://{prefix}.glovoapp.com"

    WORKING_AREA = "/b2b/working-areas"
    ORDER = "/b2b/orders"
    INVOICE = "/invoices"
    PAYMENTS = "/payments"
    REFUNDS = "/refunds"
    CARD = "/cards"
    CUSTOMER = "/customers"
    TRANSFER = "/transfers"
    VIRTUAL_ACCOUNT = "/virtual_accounts"
    SUBSCRIPTION = "/subscriptions"
    ADDON = "/addons"
    PLAN = "/plans"
    SETTLEMENT = "/settlements"



class Resource:
    endpoint = None

    def __init__(self, client=None):
        self.client = client

    def _get(self, path, data, **kwargs):
        return self.client.get(path, data, **kwargs)

    # def _patch(self, path, data, **kwargs):
    #     return self.client.patch(path, data, **kwargs)

    def _post(self, path, data, **kwargs):
        return self.client.post(path, data, **kwargs)

    # def _put(self, path, data, **kwargs):
    #     return self.client.put(path, data, **kwargs)

    # def _delete(self, path, data, **kwargs):
    #     return self.client.delete(path, data, **kwargs)

    def create(self, data, **options):
        path = self.endpoint
        return self._post(path, data, **options)

    def list(self, data=None, **options):
        path = self.endpoint
        return self._get(path, data, **options)

    def read(self, id_, data=None, **options):
        path = "{0}/{1}".format(self.endpoint, id_)
        return self._get(path, data, **options)

    # def update(self, id_, data=None, **options):
    #     path = "{0}/{1}".format(self.endpoint, id_)
    #     return self._put(path, data, **options)

    # def delete(self, id_, data=None, **options):
    #     path = "{0}/{1}".format(self.endpoint, id_)
    #     return self.delete(path, data, **options)



from .error_codes import ErrorCode
from .stages import Stage
from .status_codes import HttpStatusCode
from .urls import URL

__all__ = [
    "Stage",
    "URL",
    "HttpStatusCode",
    "ErrorCode",
]



from ..constants.urls import URL
from .base import Resource


class Order(Resource):
    endpoint = URL.ORDER

    def estimate(self, data=None, **options):
        path = "{0}/estimate".format(self.endpoint)
        return self._post(path, data, **options)

    def tracking(self, id_, data=None, **options):
        path = "{0}/{1}/tracking".format(self.endpoint, id_)
        return self._get(path, data, **options)

    def glover_info(self, id_, data=None, **options):
        path = "{0}/{1}/courier-contact".format(self.endpoint, id_)
        return self._get(path, data, **options)

    def cancel(self, id_, data=None, **options):
        path = "{0}/{1}/cancel".format(self.endpoint, id_)
        return self._post(path, data, **options)



import json
from base64 import b64encode
from types import ModuleType

from requests import session

from . import resources
from .constants import URL, Stage
from .utils import capitalize_camel_case
from .version import VERSION

RESOURCE_PREFIX = "_resource_"
RESOURCE_CLASSES = {}

for name, module in resources.__dict__.items():
    capitalized_name = capitalize_camel_case(name)
    is_module = isinstance(module, ModuleType)
    is_in_module = capitalized_name in getattr(module, "__dict__", {})

    if is_module and is_in_module:
        RESOURCE_CLASSES[name] = module.__dict__[capitalized_name]


class Glovo:
    base_url = None

    def __init__(self, api_key, api_secret, stage=Stage.PRODUCTION):
        """Initialize a Glovo client object with session.

        Also includes optional auth handler and options.
        """
        self.api_key = api_key
        self.api_secret = api_secret
        self.stage = stage
        self.session = session()

        self._set_auth_string()
        self._set_client_headers()
        self._set_base_url()

        for name, klass in RESOURCE_CLASSES.items():
            setattr(self, RESOURCE_PREFIX + name, klass(self))

    @staticmethod
    def _get_version():
        return ".".join(VERSION)

    @staticmethod
    def _update_request(data, options):
        """Update The resource data and header options."""
        data = json.dumps(data)

        if "headers" not in options:
            options["headers"] = {}

        options["headers"].update({"Content-type": "application/json"})

        return data, options

    def _set_auth_string(self):
        raw_auth_string = "{0}:{1}".format(self.api_key, self.api_secret).encode(
            "utf-8"
        )
        self.auth_string = b64encode(raw_auth_string).decode("utf-8")

    def _set_client_headers(self):
        self.session.headers.update(
            {
                "User-Agent": "Globo-API-Python/{0}".format(self._get_version()),
                "Authorization": "Basic {0}".format(self.auth_string),
                "Content-type": "application/json",
                "Accept": "application/json",
            }
        )

    def _set_base_url(self):
        prefix = URL.PREFIX[self.stage]
        self.base_url = URL.BASE_FORMAT.format(prefix=prefix)

    def _set_stage(self, stage):
        self.stage = stage
        self._set_base_url()

    def enable_test_mode(self):
        self._set_stage(Stage.TEST)

    def disable_test_mode(self):
        self._set_stage(Stage.PRODUCTION)

    def request(self, method, path, **options):
        """Dispatch a request to the Glovo HTTP API."""
        url = "{}{}".format(self.base_url, path)
        response = getattr(self.session, method)(url, **options)
        json_response = response.json()

        return {"status": response.status_code, "data": json_response}

    def get(self, path, params, **options):
        """Parse GET request options and dispatches a request."""
        return self.request("get", path, params=params, **options)

    # PATCH method is never used on Glovo resources
    # def patch(self, path, data, **options):
    #     """Parse PATCH request options and dispatches a request."""
    #     data, options = self._update_request(data, options)
    #     return self.request("patch", path, data=data, **options)

    def post(self, path, data, **options):
        """Parse POST request options and dispatches a request."""
        data, options = self._update_request(data, options)
        return self.request("post", path, data=data, **options)

    # DELETE method is never used on Glovo resources
    # def delete(self, path, data, **options):
    #     """Parse DELETE request options and dispatches a request."""
    #     data, options = self._update_request(data, options)
    #     return self.request("delete", path, data=data, **options)

    # PUT method is never used on Glovo resources
    # def put(self, path, data, **options):
    #     """Parse PUT request options and dispatches a request."""
    #     data, options = self._update_request(data, options)
    #     return self.request("put", path, data=data, **options)

    def __getattr__(self, name):
        # This method will be called if the standar accesos for a property
        # named `name` fails. I this situation if the propery name not start
        # with `RESOURCE_PREFIX` ...
        if not name.startswith(RESOURCE_PREFIX):
            # ... we will try to get the prefixed version of the attribute
            # name
            return getattr(self, RESOURCE_PREFIX + name)

        return super(Glovo, self).__getattribute__(name)
